#ifndef LIBATTR_H
#define LIBATTR_H

#include <stdio.h>
#include "LibFileInfo.h"
#include "LibVal.h"
#include "LibParse.h"

enum LibAttrType
{
  LIBERTY__SIMPLE,
  LIBERTY__COMPLEX
};

class LibGroup;

class LibAttr : public UEListNode
{
public:
 
  LibAttr() : d_type(LIBERTY__SIMPLE), d_typeID(0), d_owner(NULL) {}

  LibAttr(const UString& n, LibAttrType t, uint_t ID, LibGroup* parent,
	  const LibFileInfo& f) :
    d_name(n), d_type(t), d_typeID(ID), d_owner(parent), d_info(f) {}

  ~LibAttr();

  void               value(LibVal *v);
  void               valueList(LibValEList* vl);
  void               fileInfo(const LibFileInfo& f) { d_info = f; }

  LibAttrType        type()                   const { return d_type; }
  const UString&     name()                   const { return d_name; }
  LibVal*            value()                  const;
  LibValEList*       valueList()              const;
  uint_t             typeID()                 const { return d_typeID; }
  int                lineno()                 const { return d_info.lineno(); }
  const char *       filename()               const { return LibParse::filename();}
  LibGroup*          owner()                  const { return d_owner; }
  inline LibValIter  valueIter();
  LibVal*            firstValue()                   { return valueIter(); }
  void               dump(FILE* f, const char* indent);
  int                valueCount();

private:

  UString            d_name;
  LibAttrType        d_type;
  union {
    LibVal*          value;
    LibValEList*     vlist; // need a pointer here so we can get list from
                            // LibHead directly without copying
  }                  d_u;
  uint_t             d_typeID;
  LibGroup*          d_owner;
  LibFileInfo        d_info;

};

inline LibVal*
LibAttr::value() const
{
  uassert (d_type == LIBERTY__SIMPLE);
  return d_u.value;
}

inline void
LibAttr::value(LibVal *v)
{
  uassert (d_type == LIBERTY__SIMPLE);
  d_u.value = v;
}

inline void 
LibAttr::valueList(LibValEList* vl)
{
  uassert (d_type == LIBERTY__COMPLEX);
  d_u.vlist = vl;
}

inline LibValEList*
LibAttr::valueList() const
{
  uassert (d_type == LIBERTY__COMPLEX);
  return d_u.vlist;
}

inline LibValIter
LibAttr::valueIter()
{
  uassert (d_type == LIBERTY__COMPLEX);
  return d_u.vlist->begin();
}

template <> struct UEKey<LibAttr*>
{
  const UString& operator()(LibAttr* attr) const {
    return attr->name();
  }
};

inline int
LibAttr::valueCount()
{
  if (d_type == LIBERTY__SIMPLE) {
    return 1;
  } else {
    return d_u.vlist->size();
  }
}
    
#endif

